#include "sequence_compare/hamming_distance.h"

#include "absl/time/clock.h"
#include "absl/time/time.h"

namespace hamming_distance {
std::vector<int> WindowHammingDistance(const std::string& seq_1,
                                       const std::string& seq_2,
                                       const int window_size,
                                       const int step_size,
                                       const bool append_sites_compared) {
  CHECK_GT(window_size, 0);
  CHECK_GT(step_size, 0);
  int sites_compared;
  const std::vector<int> sparse_distance =
      CompareSequences(seq_1, seq_2, &sites_compared);
  std::vector<int>::const_iterator start_it = sparse_distance.begin(),
                                   end_it = sparse_distance.begin();
  std::vector<int> window_hamming_distance;
  int distance = 0;
  for (int window_start = 0;
       window_start + window_size <= static_cast<int>(seq_1.size());
       window_start += step_size) {
    int window_end = window_start + window_size;
    while (start_it != sparse_distance.end() && *start_it < window_start) {
      --distance;
      ++start_it;
    }
    while (end_it != sparse_distance.end() && *end_it < window_end) {
      ++distance;
      ++end_it;
    }
    CHECK_GE(distance, 0);
    window_hamming_distance.push_back(distance);
  }
  if (append_sites_compared) window_hamming_distance.push_back(sites_compared);

  return window_hamming_distance;
}

void CompareWorker(const std::vector<std::pair<std::string, std::string>>& data,
                   const int start, const int end, const int window_size,
                   const int step_size, const bool append_sites_compared,
                   dist_data* hamming_dist) {
  CHECK(hamming_dist);
  hamming_dist->clear();

  constexpr int kLogFrequency = 100000;
  int count = 0;
  absl::Time start_t = absl::Now();

  for (int i = start; i < end; ++i) {
    for (int j = 0; j < i; ++j) {
      hamming_dist->push_back(std::make_pair(
          std::make_pair(absl::string_view(data[i].first),
                         absl::string_view(data[j].first)),
          WindowHammingDistance(data[i].second, data[j].second, window_size,
                                step_size, append_sites_compared)));
      ++count;
      if (count % kLogFrequency == 0) {
        LOG(INFO) << "Worker" << start << ":" << end << " has processed "
                  << count / 1000000.0f << " m pairs ("
                  << count /
                         ((static_cast<float>(end) * end - start * start) / 2) *
                         100
                  << " %). Time elapse: "
                  << absl::ToDoubleSeconds(absl::Now() - start_t) << " seconds";
      }
    }
  }
  LOG(INFO) << "Worker" << start << ":" << end << " used "
            << absl::ToDoubleSeconds(absl::Now() - start_t) << " seconds";
}

}  // namespace hamming_distance
